#!/usr/bin/python2
import json
import socket
import os
import config

class Socket(object):
    def __init__(self, address):
        self.address = address
        self.socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        self.active = False

    def __enter__(self):
        self.socket.connect(self.address)
        self.active = True
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        if exc_type is not None:
            print exc_type, exc_value, traceback
        self.active = False
        return self

    def step(self):
        connection, client_address = self.socket.accept()
        message = self.receive(connection)
        connection.close()
        return message

    def receive(self, connection):
        parts = []
        while True:
            part = connection.recv(BUFFER_SIZE)
            if len(part) > 0:
                parts.append(part)
            else:
                break
        return json.loads(b"".join(parts).decode("utf-8"))

    def send(self, message):
        self.socket.send(json.dumps(message).encode("utf-8"))

def main():
    import sys
    if len(sys.argv) < 2:
        raise Exception("Missing arguments")
    message = {}
    print(sys.argv)
    for argument in sys.argv[1:]:
        if "=" in argument:
            key, value = argument.split("=", 1)
            message[key] = value
    with Socket(config.SOCKET_PATH) as sock:
        sock.send(message)

                
if __name__ == "__main__":
    main()
