#!/usr/bin/python


import logging, time, sys, socket, os
import subprocess

sys.path.append("/root/pyproxmox/src/")
from pyproxmox import *
logging.basicConfig(format='%(levelname)s:%(message)s', level=logging.DEBUG)

critical_infra = set([104, 105, 106, 108, 112]) # mail, admin, fsmuw, lists, vpn
hostname = socket.gethostname()

proxmox_auth = prox_auth(hostname+'.fsmpi.rwth-aachen.de', 'apc@pve', '')

pr = pyproxmox(proxmox_auth)


def shutdown( host, vmid ):
	pr.shutdownVirtualMachine(host, vmid)
	return

def migrate( host, vmid, target ):
	pr.migrateVirtualMachine(host, vmid, target, True)
	return

def stop( host, vmid ):
	pr.stopVirtualMachine(host, vmid)
	return

def start( vmid ):
	pr.startVirtualMachine(host, vmid)
	return

def getVMs( host ):
	virt_index_list = pr.getNodeVirtualIndex(host)["data"]
	return virt_index_list

def getHostStatus( host ):
	cluster = pr.getClusterStatus()
	for i in cluster:
		for j in cluster[i]:
			if j["type"] == "node":
				if j["name"] == host:
					return bool(j["state"])
				else:
					return False
	return False

def migrateCriticalVMs(dest_host, force=False):
	vms = getVMs(hostname)
	call_again = False
	for vm in vms:
		if vm['status'] == 'running':
			if vm['vmid'] in critical_infra:
				if hostname == dest_host:
					logging.info("let vm {0} ({1}) run on {2}".format(vm['vmid'], vm['name'], hostname))
				else:
					call_again = True
					logging.info("migrate vm {0} ({1}) from {2} to {3}".format(vm['vmid'], vm['name'], hostname, dest_host))
					migrate(hostname, vm['vmid'], dest_host)
			else:
				call_again = True
				if force:
					logging.info("stop vm {0} ({1}) running on {2}".format(vm['vmid'], vm['name'], hostname))
					stop(hostname, vm['vmid'])
				else:
					logging.info("shutdown vm {0} ({1}) running on {2}".format(vm['vmid'], vm['name'], hostname))
					shutdown(hostname, vm['vmid'])
	return call_again

# issue vm migration and shutdown
other_hostname = "vm01" if hostname == "vm02" else "vm01"
#if getHostStatus(other_hostname):
for i in range(0, 3):
	time.sleep(120*i)
	force = True if i > 1 else False
	logging.info("migrating critical vms from {0} to {1} (force: {2}).".format(hostname, "vm02", str(force)))
	call_again = migrateCriticalVMs("vm02", force)
	if not call_again:
		break
#else:
#	logging.warn("{0} is offline. vm migration is impossible".format(other_hostname))

vmsonhost = getVMs(hostname)
setvms = set()
for i in vmsonhost:
	setvms.add(i['vmid'])

# issue shutdown
if not len(setvms & critical_infra):
	logging.info("no critical infra left. {0} may shutdown now.".format(hostname))
	subprocess.call(["shutdown", "-h", "now"])
else:
	logging.info("this host serves critical infra. {0} shall not halt.".format(hostname))

